@@ -20,11 +20,14 @@ OVN_SG_EXT_ID_KEY = 'neutron:security_group_id'
 OVN_SG_RULE_EXT_ID_KEY = 'neutron:security_group_rule_id'
 OVN_ML2_MECH_DRIVER_NAME = 'ovn'
 OVN_NETWORK_NAME_EXT_ID_KEY = 'neutron:network_name'
+OVN_NETWORK_MTU_EXT_ID_KEY = 'neutron:mtu'
 OVN_PORT_NAME_EXT_ID_KEY = 'neutron:port_name'
+OVN_PORT_FIP_EXT_ID_KEY = 'neutron:port_fip'
 OVN_ROUTER_NAME_EXT_ID_KEY = 'neutron:router_name'
 OVN_ROUTER_IS_EXT_GW = 'neutron:is_ext_gw'
 OVN_GW_PORT_EXT_ID_KEY = 'neutron:gw_port_id'
 OVN_SUBNET_EXT_ID_KEY = 'neutron:subnet_id'
+OVN_SUBNET_EXT_IDS_KEY = 'neutron:subnet_ids'
 OVN_PHYSNET_EXT_ID_KEY = 'neutron:provnet-physical-network'
 OVN_NETTYPE_EXT_ID_KEY = 'neutron:provnet-network-type'
 OVN_SEGID_EXT_ID_KEY = 'neutron:provnet-segmentation-id'
@@ -33,27 +36,35 @@ OVN_DEVID_EXT_ID_KEY = 'neutron:device_id'
 OVN_CIDRS_EXT_ID_KEY = 'neutron:cidrs'
 OVN_FIP_EXT_ID_KEY = 'neutron:fip_id'
 OVN_FIP_PORT_EXT_ID_KEY = 'neutron:fip_port_id'
+OVN_FIP_EXT_MAC_KEY = 'neutron:fip_external_mac'
 OVN_REV_NUM_EXT_ID_KEY = 'neutron:revision_number'
 OVN_QOS_POLICY_EXT_ID_KEY = 'neutron:qos_policy_id'
 OVN_SG_IDS_EXT_ID_KEY = 'neutron:security_group_ids'
 OVN_DEVICE_OWNER_EXT_ID_KEY = 'neutron:device_owner'
+OVN_LIVENESS_CHECK_EXT_ID_KEY = 'neutron:liveness_check_at'
 OVN_PORT_BINDING_PROFILE = portbindings.PROFILE
 OVN_PORT_BINDING_PROFILE_PARAMS = [{'parent_name': six.string_types,
                                     'tag': six.integer_types},
                                    {'vtep-physical-switch': six.string_types,
                                     'vtep-logical-switch': six.string_types}]
+MIGRATING_ATTR = 'migrating_to'
 OVN_ROUTER_PORT_OPTION_KEYS = ['router-port', 'nat-addresses']
 OVN_GATEWAY_CHASSIS_KEY = 'redirect-chassis'
+OVN_CHASSIS_REDIRECT = 'chassisredirect'
 OVN_GATEWAY_NAT_ADDRESSES_KEY = 'nat-addresses'
+OVN_DROP_PORT_GROUP_NAME = 'neutron_pg_drop'
+OVN_ROUTER_PORT_GW_MTU_OPTION = 'gateway_mtu'
 
 OVN_PROVNET_PORT_NAME_PREFIX = 'provnet-'
 
-OVN_NEUTRON_OWNER_TO_PORT_TYPE = {const.DEVICE_OWNER_DHCP: 'localport'}
-
 # Agent extension constants
 OVN_AGENT_DESC_KEY = 'neutron:description'
+OVN_AGENT_METADATA_SB_CFG_KEY = 'neutron:ovn-metadata-sb-cfg'
+OVN_AGENT_METADATA_DESC_KEY = 'neutron:description-metadata'
+OVN_AGENT_METADATA_ID_KEY = 'neutron:ovn-metadata-id'
 OVN_CONTROLLER_AGENT = 'OVN Controller agent'
 OVN_CONTROLLER_GW_AGENT = 'OVN Controller Gateway agent'
+OVN_METADATA_AGENT = 'OVN Metadata agent'
 
 # OVN ACLs have priorities.  The highest priority ACL that matches is the one
 # that takes effect.  Our choice of priority numbers is arbitrary, but it
@@ -83,6 +94,14 @@ SUPPORTED_DHCP_OPTS = {
     6: ['server-id', 'dns-server', 'domain-search']}
 DHCPV6_STATELESS_OPT = 'dhcpv6_stateless'
 
+# When setting global DHCP options, these options will be ignored
+# as they are required for basic network functions and will be
+# set by Neutron.
+GLOBAL_DHCP_OPTS_BLACKLIST = {
+    4: ['server_id', 'lease_time', 'mtu', 'router', 'server_mac',
+        'dns_server', 'classless_static_route'],
+    6: ['dhcpv6_stateless', 'dns_server', 'server_id']}
+
 CHASSIS_DATAPATH_NETDEV = 'netdev'
 CHASSIS_IFACE_DPDKVHOSTUSER = 'dpdkvhostuser'
 
@@ -132,3 +151,25 @@ MAINTENANCE_CREATE_UPDATE_TYPE_ORDER = {
 # task: Leaf ones first and roots at the end.
 MAINTENANCE_DELETE_TYPE_ORDER = {
     t: n for n, t in enumerate(reversed(_TYPES_PRIORITY_ORDER), 1)}
+
+# The addresses field to set in the logical switch port which has a
+# peer router port (connecting to the logical router).
+DEFAULT_ADDR_FOR_LSP_WITH_PEER = 'router'
+
+# Loadbalancer constants
+LRP_PREFIX = "lrp-"
+LB_VIP_PORT_PREFIX = "ovn-lb-vip-"
+
+# Hash Ring constants
+HASH_RING_NODES_TIMEOUT = 60
+HASH_RING_TOUCH_INTERVAL = 30
+HASH_RING_CACHE_TIMEOUT = 30
+HASH_RING_ML2_GROUP = 'mechanism_driver'
+
+# Maximum chassis count where a gateway port can be hosted
+MAX_GW_CHASSIS = 5
+
+UNKNOWN_ADDR = 'unknown'
+
+# TODO(lucasagomes): Create constants for other LSP types
+LSP_TYPE_LOCALNET = 'localnet'
