@@ -15,7 +15,7 @@ import os
 import re
 
 import netaddr
-
+from neutron_lib.api.definitions import external_net
 from neutron_lib.api.definitions import extra_dhcp_opt as edo_ext
 from neutron_lib.api.definitions import l3
 from neutron_lib.api.definitions import port_security as psec
@@ -55,7 +55,7 @@ def ovn_lrouter_port_name(id):
     #   - patch-lrp-<UUID>-to-<UUID>
     #   - patch-<UUID>-to-lrp-<UUID>
     # lrp stands for Logical Router Port
-    return 'lrp-%s' % id
+    return constants.LRP_PREFIX + '%s' % id
 
 
 def ovn_provnet_port_name(network_id):
@@ -334,7 +334,7 @@ def get_lrouter_non_gw_routes(ovn_router):
         for route in getattr(ovn_router, 'static_routes', []):
             external_ids = getattr(route, 'external_ids', {})
             if strutils.bool_from_string(
-                external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
+                    external_ids.get(constants.OVN_ROUTER_IS_EXT_GW, 'false')):
                 continue
 
             routes.append({'destination': route.ip_prefix,
@@ -389,3 +389,33 @@ def get_method_class(method):
 def ovn_metadata_name(id_):
     """Return the OVN metadata name based on an id."""
     return 'metadata-%s' % id_
+
+
+def is_gateway_chassis_invalid(chassis_name, gw_chassis,
+                               physnet, chassis_physnets):
+    """Check if gateway chassis is invalid
+
+    @param    chassis_name: gateway chassis name
+    @type     chassis_name: string
+    @param    gw_chassis: List of gateway chassis in the system
+    @type     gw_chassis: []
+    @param    physnet: physical network associated to chassis_name
+    @type     physnet: string
+    @param    chassis_physnets: Dictionary linking chassis with their physnets
+    @type     chassis_physnets: {}
+    @return   Boolean
+    """
+
+    if chassis_name == constants.OVN_GATEWAY_INVALID_CHASSIS:
+        return True
+    elif chassis_name not in chassis_physnets:
+        return True
+    elif physnet and physnet not in chassis_physnets.get(chassis_name):
+        return True
+    elif gw_chassis and chassis_name not in gw_chassis:
+        return True
+    return False
+
+
+def is_provider_network(network):
+    return external_net.EXTERNAL in network
