@@ -10,6 +10,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from oslo_utils import timeutils
 from ovsdbapp.backend.ovs_idl import command
 from ovsdbapp.backend.ovs_idl import idlutils
 
@@ -85,6 +86,19 @@ def _add_gateway_chassis(api, txn, lrp_name, val):
         return 'options', chassis
 
 
+class CheckLivenessCommand(command.BaseCommand):
+    def __init__(self, api):
+        super(CheckLivenessCommand, self).__init__(api)
+
+    def run_idl(self, txn):
+        # txn.pre_commit responsible for updating nb_global.nb_cfg, but
+        # python-ovs will not update nb_cfg if no other changes are made
+        self.api.nb_global.setkey('external_ids',
+                                  ovn_const.OVN_LIVENESS_CHECK_EXT_ID_KEY,
+                                  str(timeutils.utcnow(with_timezone=True)))
+        self.result = self.api.nb_global.nb_cfg
+
+
 class LSwitchSetExternalIdsCommand(command.BaseCommand):
     def __init__(self, api, name, ext_ids, if_exists):
         super(LSwitchSetExternalIdsCommand, self).__init__(api)
@@ -396,12 +410,14 @@ class DelLRouterPortCommand(command.BaseCommand):
 
 
 class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
-    def __init__(self, api, lswitch_port, lrouter_port, is_gw_port, if_exists):
+    def __init__(self, api, lswitch_port, lrouter_port, is_gw_port,
+                 if_exists, lsp_address):
         super(SetLRouterPortInLSwitchPortCommand, self).__init__(api)
         self.lswitch_port = lswitch_port
         self.lrouter_port = lrouter_port
         self.is_gw_port = is_gw_port
         self.if_exists = if_exists
+        self.lsp_address = lsp_address
 
     def run_idl(self, txn):
         try:
@@ -419,7 +435,7 @@ class SetLRouterPortInLSwitchPortCommand(command.BaseCommand):
             options[ovn_const.OVN_GATEWAY_NAT_ADDRESSES_KEY] = 'router'
         setattr(port, 'options', options)
         setattr(port, 'type', 'router')
-        setattr(port, 'addresses', 'router')
+        setattr(port, 'addresses', self.lsp_address)
 
 
 class AddACLCommand(command.BaseCommand):
@@ -588,17 +604,17 @@ class UpdateACLsCommand(command.BaseCommand):
 
         if self.need_compare:
             # Get all relevant ACLs in 1 shot
-            acl_values_dict, acl_obj_dict, lswitch_ovsdb_dict = \
-                self.api.get_acls_for_lswitches(self.lswitch_names)
+            acl_values_dict, acl_obj_dict, lswitch_ovsdb_dict = (
+                self.api.get_acls_for_lswitches(self.lswitch_names))
 
             # Compute the difference between the new and old set of ACLs
-            acl_del_objs_dict, acl_add_values_dict = \
+            acl_del_objs_dict, acl_add_values_dict = (
                 self._compute_acl_differences(
                     self.port_list, acl_values_dict,
-                    self.acl_new_values_dict, acl_obj_dict)
+                    self.acl_new_values_dict, acl_obj_dict))
         else:
-            lswitch_ovsdb_dict, acl_del_objs_dict, acl_add_values_dict = \
-                self._get_update_data_without_compare()
+            lswitch_ovsdb_dict, acl_del_objs_dict, acl_add_values_dict = (
+                self._get_update_data_without_compare())
 
         for lswitch_name, lswitch in lswitch_ovsdb_dict.items():
             acl_del_objs = acl_del_objs_dict.get(lswitch_name, [])
@@ -922,9 +938,9 @@ class DeleteNATRuleInLRouterCommand(command.BaseCommand):
             type = getattr(nat, 'type', '')
             external_ip = getattr(nat, 'external_ip', '')
             logical_ip = getattr(nat, 'logical_ip', '')
-            if self.type == type and \
-               self.external_ip == external_ip and \
-               self.logical_ip == logical_ip:
+            if (self.type == type and
+                    self.external_ip == external_ip and
+                    self.logical_ip == logical_ip):
                 nats.remove(nat)
                 nat.delete()
                 break
@@ -955,71 +971,6 @@ class SetNATRuleInLRouterCommand(command.BaseCommand):
                 break
 
 
-class AddNatIpToLRPortPeerOptionsCommand(command.BaseCommand):
-    # TODO(chandrav): Add unit tests, bug #1638715.
-    def __init__(self, api, lport, nat_ip):
-        super(AddNatIpToLRPortPeerOptionsCommand, self).__init__(api)
-        self.lport = lport
-        self.nat_ip = nat_ip
-
-    def run_idl(self, txn):
-        try:
-            lport = idlutils.row_by_value(self.api.idl, 'Logical_Switch_Port',
-                                          'name', self.lport)
-        except idlutils.RowNotFound:
-            msg = _("Logical Switch Port %s does not exist") % self.lport
-            raise RuntimeError(msg)
-
-        lport.verify('addresses')
-        addresses = getattr(lport, 'addresses', [])
-        lport.verify('options')
-        options = getattr(lport, 'options', {})
-
-        nat_ip_list = options.get('nat-addresses', '').split()
-        if not nat_ip_list:
-            nat_ips = addresses[0].split()[0] + ' ' + self.nat_ip
-        elif self.nat_ip not in nat_ip_list:
-            nat_ip_list.append(self.nat_ip)
-            nat_ips = ' '.join(nat_ip_list)
-        else:
-            return
-        options['nat-addresses'] = nat_ips
-        lport.options = options
-
-
-class DeleteNatIpFromLRPortPeerOptionsCommand(command.BaseCommand):
-    # TODO(chandrav): Add unit tests, bug #1638715.
-    def __init__(self, api, lport, nat_ip):
-        super(DeleteNatIpFromLRPortPeerOptionsCommand, self).__init__(api)
-        self.lport = lport
-        self.nat_ip = nat_ip
-
-    def run_idl(self, txn):
-        try:
-            lport = idlutils.row_by_value(self.api.idl, 'Logical_Switch_Port',
-                                          'name', self.lport)
-
-        except idlutils.RowNotFound:
-            msg = _("Logical Switch Port %s does not exist") % self.lport
-            raise RuntimeError(msg)
-
-        lport.verify('options')
-        options = getattr(lport, 'options', {})
-        nat_ip_list = options.get('nat-addresses', '').split()
-        if self.nat_ip not in nat_ip_list:
-            return
-        nat_ip_list.remove(self.nat_ip)
-        if len(nat_ip_list) is 1:
-            nat_ips = ''
-        else:
-            nat_ips = ' '.join(nat_ip_list)
-        if not nat_ips:
-            del options['nat-addresses']
-        else:
-            options['nat-addresses'] = nat_ips
-        lport.options = options
-
-
 class CheckRevisionNumberCommand(command.BaseCommand):
 
     def __init__(self, api, name, resource, resource_type, if_exists):
