@@ -16,15 +16,21 @@
 import mock
 
 from neutron.common import utils as n_utils
+from neutron_lib.plugins import directory
 from octavia_lib.api.drivers import data_models as octavia_data_model
+from octavia_lib.api.drivers import driver_lib
 from octavia_lib.api.drivers import exceptions as o_exceptions
 from octavia_lib.common import constants as o_constants
 from oslo_serialization import jsonutils
 from oslo_utils import uuidutils
+from ovsdbapp.schema.ovn_northbound import impl_idl as idl_ovn
 
+from networking_ovn.common import constants as ovn_const
 from networking_ovn.octavia import ovn_driver
 from networking_ovn.tests.functional import base
 
+LR_REF_KEY_HEADER = 'neutron-'
+
 
 class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
 
@@ -33,7 +39,18 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         # ovn_driver.OvnProviderHelper.ovn_nbdb_api is a class variable.
         # Set it to None, so that when a worker starts the 2nd test we don't
         # use the old object.
+        idl_ovn.OvnNbApiIdlImpl.ovsdb_connection = None
         ovn_driver.OvnProviderHelper.ovn_nbdb_api = None
+        ovn_driver.OvnProviderHelper.ovn_nbdb_api_for_events = None
+        # TODO(mjozefcz): Use octavia listeners to provide needed
+        # sockets and modify tests in order to verify if fake
+        # listener (status) has received valid value.
+        try:
+            mock.patch.object(
+                driver_lib.DriverLibrary, '_check_for_socket_ready').start()
+        except AttributeError:
+            # Backward compatiblity with octavia-lib < 1.3.1
+            pass
         self.ovn_driver = ovn_driver.OvnProviderDriver()
         self.ovn_driver._ovn_helper._octavia_driver_lib = mock.MagicMock()
         self._o_driver_lib = self.ovn_driver._ovn_helper._octavia_driver_lib
@@ -44,8 +61,12 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         self.fake_network_driver.get_subnet = self._mock_get_subnet
         self.fake_network_driver.neutron_client.list_ports = (
             self._mock_list_ports)
+        self.fake_network_driver.neutron_client.\
+            delete_port.return_value = True
         self._local_net_cache = {}
         self._local_port_cache = {'ports': []}
+        self.addCleanup(self.ovn_driver._ovn_helper.shutdown)
+        self.core_plugin = directory.get_plugin()
 
     def _mock_get_subnet(self, subnet_id):
         m_subnet = mock.MagicMock()
@@ -55,7 +76,24 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
     def _mock_list_ports(self, **kwargs):
         return self._local_port_cache
 
+    def _create_provider_network(self):
+        e1 = self._make_network(self.fmt, 'e1', True,
+                                arg_list=('router:external',
+                                          'provider:network_type',
+                                          'provider:physical_network'),
+                                **{'router:external': True,
+                                   'provider:network_type': 'flat',
+                                   'provider:physical_network': 'public'})
+        res = self._create_subnet(self.fmt, e1['network']['id'],
+                                  '100.0.0.0/24', gateway_ip='100.0.0.254',
+                                  allocation_pools=[{'start': '100.0.0.2',
+                                                     'end': '100.0.0.253'}],
+                                  enable_dhcp=False)
+        e1_s1 = self.deserialize(self.fmt, res)
+        return e1, e1_s1
+
     def _create_lb_model(self, vip=None, vip_network_id=None,
+                         vip_port_id=None,
                          admin_state_up=True):
         lb = octavia_data_model.LoadBalancer()
         lb.loadbalancer_id = uuidutils.generate_uuid()
@@ -67,11 +105,16 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
 
         if vip_network_id:
             lb.vip_network_id = vip_network_id
+        if vip_port_id:
+            lb.vip_port_id = vip_port_id
         lb.admin_state_up = admin_state_up
         return lb
 
-    def _create_pool_model(self, loadbalancer_id, pool_name, protocol=None,
-                           admin_state_up=True, listener_id=None):
+    def _create_pool_model(
+            self, loadbalancer_id, pool_name,
+            protocol=o_constants.PROTOCOL_TCP,
+            lb_algorithm=o_constants.LB_ALGORITHM_SOURCE_IP_PORT,
+            admin_state_up=True, listener_id=None):
         m_pool = octavia_data_model.Pool()
         if protocol:
             m_pool.protocol = protocol
@@ -82,6 +125,8 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         m_pool.loadbalancer_id = loadbalancer_id
         m_pool.members = []
         m_pool.admin_state_up = admin_state_up
+        m_pool.protocol = protocol
+        m_pool.lb_algorithm = lb_algorithm
         if listener_id:
             m_pool.listener_id = listener_id
         return m_pool
@@ -143,12 +188,19 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         self.assertItemsEqual(expected_lbs, observed_lbs)
 
     def _is_lb_associated_to_ls(self, lb_name, ls_name):
+        return self._is_lb_associated_to_tab(
+            'Logical_Switch', lb_name, ls_name)
+
+    def _is_lb_associated_to_lr(self, lb_name, lr_name):
+        return self._is_lb_associated_to_tab(
+            'Logical_Router', lb_name, lr_name)
+
+    def _is_lb_associated_to_tab(self, table, lb_name, ls_name):
         lb_uuid = self._get_loadbalancer_id(lb_name)
-        for ls in self.nb_api.tables['Logical_Switch'].rows.values():
+        for ls in self.nb_api.tables[table].rows.values():
             if ls.name == ls_name:
                 ls_lbs = [lb.uuid for lb in ls.load_balancer]
                 return lb_uuid in ls_lbs
-
         return False
 
     def _create_router(self, name, gw_info=None):
@@ -178,8 +230,8 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
                 port['port']['id'])
 
     def _update_ls_refs(self, lb_data, net_id, add_ref=True):
-        if not net_id.startswith("neutron-"):
-            net_id = "neutron-" + net_id
+        if not net_id.startswith(LR_REF_KEY_HEADER):
+            net_id = LR_REF_KEY_HEADER + '%s' % net_id
 
         if add_ref:
             if net_id in lb_data[ovn_driver.LB_EXT_IDS_LS_REFS_KEY]:
@@ -195,8 +247,7 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
             else:
                 del lb_data[ovn_driver.LB_EXT_IDS_LS_REFS_KEY][net_id]
 
-    def _wait_for_status_and_validate(self, lb_data, expected_status,
-                                      check_call=True):
+    def _wait_for_status(self, expected_status, check_call=True):
         call_count = len(expected_status)
         expected_calls = [mock.call(status) for status in expected_status]
         update_loadbalancer_status = (
@@ -205,33 +256,47 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
             lambda: update_loadbalancer_status.call_count == call_count,
             timeout=10)
         if check_call:
-            try:
-                self._o_driver_lib.update_loadbalancer_status.assert_has_calls(
-                    expected_calls, any_order=True)
-            except Exception:
-                raise Exception(
-                    self._o_driver_lib.update_loadbalancer_status.mock_calls)
+            self._o_driver_lib.update_loadbalancer_status.assert_has_calls(
+                expected_calls, any_order=True)
+
+    def _wait_for_status_and_validate(self, lb_data, expected_status,
+                                      check_call=True):
+        self._wait_for_status(expected_status, check_call)
         expected_lbs = self._make_expected_lbs(lb_data)
         self._validate_loadbalancers(expected_lbs)
 
     def _create_load_balancer_and_validate(self, lb_info,
-                                           admin_state_up=True):
+                                           admin_state_up=True,
+                                           only_model=False,
+                                           create_router=True,
+                                           multiple_lb=False):
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         lb_data = {}
-        router_id = self._create_router("r1")
-        lb_data[ovn_driver.LB_EXT_IDS_LR_REF_KEY] = router_id
+        r_id = self._create_router("r1") if create_router else None
+        if r_id:
+            lb_data[
+                ovn_driver.LB_EXT_IDS_LR_REF_KEY] = LR_REF_KEY_HEADER + r_id
         net_info = self._create_net(lb_info['vip_network'], lb_info['cidr'],
-                                    router_id=router_id)
+                                    router_id=r_id)
         lb_data['vip_net_info'] = net_info
         lb_data['model'] = self._create_lb_model(vip=net_info[2],
                                                  vip_network_id=net_info[0],
+                                                 vip_port_id=net_info[3],
                                                  admin_state_up=admin_state_up)
         lb_data[ovn_driver.LB_EXT_IDS_LS_REFS_KEY] = {}
         lb_data['listeners'] = []
         lb_data['pools'] = []
         self._update_ls_refs(lb_data, net_info[0])
+        if only_model:
+            return lb_data
 
         self.ovn_driver.loadbalancer_create(lb_data['model'])
+
+        name = '%s%s' % (ovn_const.LB_VIP_PORT_PREFIX,
+                         lb_data['model'].loadbalancer_id)
+        self.driver.update_port(
+            self.context, net_info[3], {'port': {'name': name}})
+
         if lb_data['model'].admin_state_up:
             expected_status = {
                 'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
@@ -244,10 +309,16 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
                                    "provisioning_status": "ACTIVE",
                                    "operating_status": o_constants.OFFLINE}]
             }
-        self._wait_for_status_and_validate(lb_data, [expected_status])
+        if not multiple_lb:
+            self._wait_for_status_and_validate(lb_data, [expected_status])
+        else:
+            l_id = lb_data['model'].loadbalancer_id
+            self._wait_for_status([expected_status])
+            self.assertIn(l_id,
+                          [lb['name'] for lb in self._get_loadbalancers()])
         self.assertTrue(
             self._is_lb_associated_to_ls(lb_data['model'].loadbalancer_id,
-                                         'neutron-' + net_info[0]))
+                                         LR_REF_KEY_HEADER + net_info[0]))
         return lb_data
 
     def _update_load_balancer_and_validate(self, lb_data,
@@ -273,7 +344,8 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
 
         self._wait_for_status_and_validate(lb_data, [expected_status])
 
-    def _delete_load_balancer_and_validate(self, lb_data, cascade=False):
+    def _delete_load_balancer_and_validate(self, lb_data, cascade=False,
+                                           multiple_lb=False):
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         self.ovn_driver.loadbalancer_delete(lb_data['model'], cascade)
         expected_status = {
@@ -300,15 +372,19 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
                     "operating_status": "OFFLINE"})
             expected_status = {
                 key: value for key, value in expected_status.items() if value}
-
+        l_id = lb_data['model'].loadbalancer_id
         lb = lb_data['model']
         del lb_data['model']
-        self._wait_for_status_and_validate(lb_data, [expected_status])
-        self._validate_loadbalancers([])
+        if not multiple_lb:
+            self._wait_for_status_and_validate(lb_data, [expected_status])
+        else:
+            self._wait_for_status([expected_status])
+            self.assertNotIn(
+                l_id, [lbs['name'] for lbs in self._get_loadbalancers()])
         vip_net_id = lb_data['vip_net_info'][0]
         self.assertFalse(
             self._is_lb_associated_to_ls(lb.loadbalancer_id,
-                                         'neutron-' + vip_net_id))
+                                         LR_REF_KEY_HEADER + vip_net_id))
 
     def _make_expected_lbs(self, lb_data):
         if not lb_data or not lb_data.get('model'):
@@ -339,14 +415,13 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
             pool_info[p.pool_id] = p_members
 
         for net_id, ref_ct in lb_data[
-            ovn_driver.LB_EXT_IDS_LS_REFS_KEY].items():
+                ovn_driver.LB_EXT_IDS_LS_REFS_KEY].items():
             external_ids[ovn_driver.LB_EXT_IDS_LS_REFS_KEY][net_id] = ref_ct
 
         if lb_data.get(ovn_driver.LB_EXT_IDS_LR_REF_KEY):
             external_ids[
-                ovn_driver.LB_EXT_IDS_LR_REF_KEY] = 'neutron-' + lb_data[
+                ovn_driver.LB_EXT_IDS_LR_REF_KEY] = lb_data[
                     ovn_driver.LB_EXT_IDS_LR_REF_KEY]
-
         expected_vips = {}
         expected_protocol = ['tcp']
         for l in lb_data['listeners']:
@@ -394,7 +469,7 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         expected_status = {
             'pools': [{'id': m_pool.pool_id,
                        'provisioning_status': 'ACTIVE',
-                       'operating_status': o_constants.ONLINE}],
+                       'operating_status': o_constants.OFFLINE}],
             'loadbalancers': [{'id': m_pool.loadbalancer_id,
                                'provisioning_status': 'ACTIVE'}]
         }
@@ -453,7 +528,8 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         for m in p.members:
             expected_status.append(
                 {'pools': [{"id": p.pool_id,
-                            "provisioning_status": "ACTIVE"}],
+                            "provisioning_status": o_constants.ACTIVE,
+                            "operating_status": o_constants.ONLINE}],
                  'members': [{"id": m.member_id,
                               "provisioning_status": "DELETED"}],
                  'loadbalancers': [{"id": p.loadbalancer_id,
@@ -461,7 +537,11 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
                  'listeners': []})
             self._update_ls_refs(
                 lb_data, self._local_net_cache[m.subnet_id], add_ref=False)
-
+        if p.members:
+            # If Pool has members, delete all members of the pool. When the
+            # last member is processed set Operating status of Pool as Offline
+            expected_status[-1]['pools'][0][
+                'operating_status'] = o_constants.OFFLINE
         pool_dict = {
             'pools': [{'id': p.pool_id,
                        'provisioning_status': 'DELETED'}],
@@ -500,6 +580,9 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
                                     network_id, address):
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
+        pool_status = {'id': pool.pool_id,
+                       'provisioning_status': o_constants.ACTIVE,
+                       'operating_status': o_constants.ONLINE}
 
         m_member = self._create_member_model(pool.pool_id, subnet_id, address)
         pool.members.append(m_member)
@@ -507,14 +590,12 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         self.ovn_driver.member_create(m_member)
         self._update_ls_refs(lb_data, network_id)
         pool_listeners = self._get_pool_listeners(lb_data, pool_id)
-
         expected_listener_status = [
             {'id': l.listener_id, 'provisioning_status': 'ACTIVE'}
             for l in pool_listeners]
 
         expected_status = {
-            'pools': [{'id': pool.pool_id,
-                       'provisioning_status': 'ACTIVE'}],
+            'pools': [pool_status],
             'members': [{"id": m_member.member_id,
                          "provisioning_status": "ACTIVE"}],
             'loadbalancers': [{'id': pool.loadbalancer_id,
@@ -538,12 +619,17 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
             'pools': [{'id': pool.pool_id,
                        'provisioning_status': 'ACTIVE'}],
             'members': [{"id": member.member_id,
-                         'provisioning_status': 'ACTIVE',
-                         'operating_status': 'ONLINE'}],
+                         'provisioning_status': 'ACTIVE'}],
             'loadbalancers': [{'id': pool.loadbalancer_id,
                                'provisioning_status': 'ACTIVE'}],
             'listeners': []
         }
+        if getattr(member, 'admin_state_up', None):
+            expected_status['members'][0]['operating_status'] = \
+                "ONLINE"
+        else:
+            expected_status['members'][0]['operating_status'] = \
+                "OFFLINE"
         self._wait_for_status_and_validate(lb_data, [expected_status])
 
     def _update_members_in_batch_and_validate(self, lb_data, pool_id,
@@ -585,11 +671,16 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         pool = self._get_pool_from_lb_data(lb_data, pool_id=pool_id)
         member = self._get_pool_member(pool, member_address)
         pool.members.remove(member)
+        pool_status = {"id": pool.pool_id,
+                       "provisioning_status": o_constants.ACTIVE,
+                       "operating_status": o_constants.ONLINE}
+        if not pool.members:
+            pool_status['operating_status'] = o_constants.OFFLINE
+
         self._o_driver_lib.update_loadbalancer_status.reset_mock()
         self.ovn_driver.member_delete(member)
         expected_status = {
-            'pools': [{"id": pool.pool_id,
-                       "provisioning_status": "ACTIVE"}],
+            'pools': [pool_status],
             'members': [{"id": member.member_id,
                          "provisioning_status": "DELETED"}],
             'loadbalancers': [{"id": pool.loadbalancer_id,
@@ -637,7 +728,6 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
             if not admin_state_up:
                 operating_status = 'OFFLINE'
         m_listener.protocol = protocol
-
         self.ovn_driver.listener_update(m_listener, m_listener)
         pool_status = [{'id': m_listener.default_pool_id,
                         'provisioning_status': 'ACTIVE'}]
@@ -685,6 +775,21 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
              'cidr': '10.0.0.0/24'}, admin_state_up=False)
         self._delete_load_balancer_and_validate(lb_data)
 
+    def test_delete_lb_on_nonexisting_lb(self):
+        # LoadBalancer doesnt exist anymore, so just create a model and delete
+        lb_data = self._create_load_balancer_and_validate(
+            {'vip_network': 'vip_network',
+             'cidr': '19.0.0.0/24'},
+            only_model=True)
+        self.ovn_driver.loadbalancer_delete(lb_data['model'])
+        expected_status = {
+            'loadbalancers': [{"id": lb_data['model'].loadbalancer_id,
+                               "provisioning_status": "DELETED",
+                               "operating_status": "OFFLINE"}]
+        }
+        del lb_data['model']
+        self._wait_for_status_and_validate(lb_data, [expected_status])
+
     def test_pool(self):
         lb_data = self._create_load_balancer_and_validate(
             {'vip_network': 'vip_network',
@@ -764,7 +869,7 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         pool_id = lb_data['pools'][0].pool_id
         self._create_member_and_validate(
             lb_data, pool_id, lb_data['vip_net_info'][1],
-            lb_data['vip_net_info'][0], '10.0.0.10')
+            lb_data['vip_net_info'][0], '10.0.0.4')
 
         net_info = self._create_net('net1', '20.0.0.0/24')
         self._create_member_and_validate(lb_data, pool_id,
@@ -778,6 +883,13 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
 
         self._delete_listener_and_validate(lb_data, 82)
         self._delete_listener_and_validate(lb_data, 80)
+        self._delete_member_and_validate(lb_data, pool_id,
+                                         net_info[0], '20.0.0.4')
+        self._delete_member_and_validate(lb_data, pool_id,
+                                         lb_data['vip_net_info'][0],
+                                         '10.0.0.4')
+        self._delete_pool_and_validate(lb_data, "p1")
+        self._delete_load_balancer_and_validate(lb_data)
 
     def _test_cascade_delete(self, pool=True, listener=True, member=True):
         lb_data = self._create_load_balancer_and_validate(
@@ -827,6 +939,122 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         self.assertRaises(o_exceptions.UnsupportedOptionError,
                           self._update_listener_and_validate,
                           lb_data, protocol_port=80, protocol='UDP')
+        self._delete_load_balancer_and_validate(lb_data)
+
+    def _test_lrp_event_handler(self, cascade=False):
+        # Create Network N1 on router R1 and LBA on N1
+        lba_data = self._create_load_balancer_and_validate(
+            {'vip_network': 'N1',
+             'cidr': '10.0.0.0/24'})
+        router_id = lba_data[ovn_driver.LB_EXT_IDS_LR_REF_KEY][
+            len(LR_REF_KEY_HEADER):]
+        # Create Network N2, connect it to R1
+        nw_info = self._create_net("N2", "10.0.1.0/24", router_id)
+
+        # Check if LBA exists in N2 LS
+        n_utils.wait_until_true(
+            lambda: self._is_lb_associated_to_ls(
+                lba_data['model'].loadbalancer_id,
+                LR_REF_KEY_HEADER + nw_info[0]),
+            timeout=10)
+
+        # Create Network N3
+        lbb_data = self._create_load_balancer_and_validate(
+            {'vip_network': 'N3',
+             'cidr': '10.0.2.0/24'}, create_router=False, multiple_lb=True)
+        # Add N3 to R1
+        self.l3_plugin.add_router_interface(
+            self.context, lba_data[
+                ovn_driver.LB_EXT_IDS_LR_REF_KEY][len(LR_REF_KEY_HEADER):],
+            {'subnet_id': lbb_data['vip_net_info'][1]})
+
+        # Check LBB exists on R1
+        n_utils.wait_until_true(
+            lambda: self._is_lb_associated_to_lr(
+                lbb_data['model'].loadbalancer_id,
+                lba_data[ovn_driver.LB_EXT_IDS_LR_REF_KEY]),
+            timeout=10)
+        # Check LBA connected to N3
+        n_utils.wait_until_true(
+            lambda: self._is_lb_associated_to_ls(
+                lba_data['model'].loadbalancer_id,
+                LR_REF_KEY_HEADER + lbb_data['vip_net_info'][0]),
+            timeout=10)
+        # Check LBB connected to N1
+        n_utils.wait_until_true(
+            lambda: self._is_lb_associated_to_ls(
+                lbb_data['model'].loadbalancer_id,
+                LR_REF_KEY_HEADER + lba_data['vip_net_info'][0]),
+            timeout=10)
+        # Check LBB connected to N2
+        n_utils.wait_until_true(
+            lambda: self._is_lb_associated_to_ls(
+                lbb_data['model'].loadbalancer_id,
+                LR_REF_KEY_HEADER + nw_info[0]),
+            timeout=10)
+
+        lbb_id = lbb_data['model'].loadbalancer_id
+        if not cascade:
+            # N3 removed from R1
+            self.l3_plugin.remove_router_interface(
+                self.context, lba_data[
+                    ovn_driver.LB_EXT_IDS_LR_REF_KEY][len(LR_REF_KEY_HEADER):],
+                {'subnet_id': lbb_data['vip_net_info'][1]})
+        else:
+            # Delete LBB Cascade
+            self._delete_load_balancer_and_validate(lbb_data, cascade=True,
+                                                    multiple_lb=True)
+
+        # Check LBB doesn't exists on R1
+        n_utils.wait_until_true(
+            lambda: not self._is_lb_associated_to_lr(
+                lbb_id, lba_data[ovn_driver.LB_EXT_IDS_LR_REF_KEY]),
+            timeout=10)
+        # Check LBB not connected to N1
+        n_utils.wait_until_true(
+            lambda: not self._is_lb_associated_to_ls(
+                lbb_id, LR_REF_KEY_HEADER + lba_data['vip_net_info'][0]),
+            timeout=10)
+        # Check LBB not connected to N2
+        n_utils.wait_until_true(
+            lambda: not self._is_lb_associated_to_ls(
+                lbb_id, LR_REF_KEY_HEADER + nw_info[0]),
+            timeout=10)
+
+    def test_lrp_event_handler_with_interface_delete(self):
+        self._test_lrp_event_handler()
+
+    def test_lrp_event_handler_with_loadbalancer_cascade_delete(self):
+        self._test_lrp_event_handler(cascade=True)
+
+    def test_lrp_event_handler_lrp_with_external_gateway(self):
+        # Create Network N1 on router R1 and LBA on N1
+        lba_data = self._create_load_balancer_and_validate(
+            {'vip_network': 'N1',
+             'cidr': '10.0.0.0/24'})
+        router_id = lba_data[ovn_driver.LB_EXT_IDS_LR_REF_KEY][
+            len(LR_REF_KEY_HEADER):]
+
+        # Create provider network N2, connect it to R1
+        provider_net, provider_subnet = self._create_provider_network()
+        self.l3_plugin.update_router(
+            self.context,
+            router_id,
+            {'router': {
+                'id': router_id,
+                'external_gateway_info': {
+                    'enable_snat': True,
+                    'network_id': provider_net['network']['id'],
+                    'external_fixed_ips': [
+                        {'ip_address': '100.0.0.2',
+                         'subnet_id': provider_subnet['subnet']['id']}]}}})
+
+        # Check if LBA doesn't exist in provider network LS
+        n_utils.wait_until_true(
+            lambda: not self._is_lb_associated_to_ls(
+                lba_data['model'].loadbalancer_id,
+                LR_REF_KEY_HEADER + provider_net['network']['id']),
+            timeout=10)
 
     def test_lb_listener_pool_workflow(self):
         lb_data = self._create_load_balancer_and_validate(
@@ -865,3 +1093,74 @@ class TestOctaviaOvnProviderDriver(base.TestOVNFunctionalBase):
         # Deleting one member, while keeping the other member available
         self._update_members_in_batch_and_validate(lb_data, pool_id,
                                                    [m_member])
+        self._delete_load_balancer_and_validate(lb_data)
+
+    def test_fip_on_lb_vip(self):
+        """This test checks if FIP on LB VIP is configured.
+
+           This test validates if Load_Balancer VIP field
+           consist Floating IP address that is configured
+           on LB VIP port.
+        """
+        # Create LB
+        lb_data = self._create_load_balancer_and_validate(
+            {'vip_network': 'vip_network',
+             'cidr': '10.0.0.0/24'})
+        # Create a pool
+        self._create_pool_and_validate(lb_data, "p1")
+        pool_id = lb_data['pools'][0].pool_id
+        # Create listener
+        self._create_listener_and_validate(lb_data, pool_id, 80)
+        # Create Member-1 and associate it with lb_data
+        self._create_member_and_validate(
+            lb_data, pool_id, lb_data['vip_net_info'][1],
+            lb_data['vip_net_info'][0], '10.0.0.10')
+
+        # Create provider network.
+        e1, e1_s1 = self._create_provider_network()
+
+        # Configure external_gateway for router
+        router_id = lb_data['lr_ref'][8::]
+        self.l3_plugin.update_router(
+            self.context,
+            router_id,
+            {'router': {
+                'id': router_id,
+                'external_gateway_info': {
+                    'enable_snat': True,
+                    'network_id': e1['network']['id'],
+                    'external_fixed_ips': [
+                        {'ip_address': '100.0.0.2',
+                         'subnet_id': e1_s1['subnet']['id']}]}}})
+
+        # Create floating IP on LB VIP port
+        vip_port_id = lb_data['model'].vip_port_id
+        vip_port = self.core_plugin.get_ports(
+            self.context, filters={'id': [vip_port_id]})[0]
+        self.l3_plugin.create_floatingip(
+            self.context, {'floatingip': {
+                'tenant_id': self._tenant_id,
+                'floating_network_id': e1['network']['id'],
+                'subnet_id': None,
+                'floating_ip_address': '100.0.0.20',
+                'port_id': vip_port['id']}})
+
+        # Validate if FIP is stored as VIP in LB
+        lbs = self._get_loadbalancers()
+        expected_vips = {
+            '%s:80' % vip_port['fixed_ips'][0]['ip_address']: '10.0.0.10:80',
+            '100.0.0.20:80': '10.0.0.10:80'}
+        self.assertDictEqual(expected_vips,
+                             lbs[0].get('vips'))
+
+        provider_net = 'neutron-%s' % e1['network']['id']
+        tenant_net = 'neutron-%s' % lb_data['model'].vip_network_id
+        for ls in self.nb_api.tables['Logical_Switch'].rows.values():
+            if ls.name == tenant_net:
+                # Make sure that LB1 is added to tenant network
+                self.assertIn(
+                    lb_data['model'].loadbalancer_id,
+                    [lb.name for lb in ls.load_balancer])
+            elif ls.name == provider_net:
+                # Make sure that LB1 is not added to provider net - e1 LS
+                self.assertListEqual([], ls.load_balancer)
